package com.gic.haoban.manage.service.service.chat.impl;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.BasePageInfo;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.commons.util.UniqueIdUtils;
import com.gic.haoban.manage.api.dto.SecretSettingDTO;
import com.gic.haoban.manage.api.dto.StaffClerkRelationDTO;
import com.gic.haoban.manage.api.dto.WxEnterpriseQwDTO;
import com.gic.haoban.manage.api.dto.chat.GroupChatOwnerDTO;
import com.gic.haoban.manage.api.dto.chat.GroupChatOwnerHistoryDTO;
import com.gic.haoban.manage.api.dto.qdto.chat.GroupChatSearchQDTO;
import com.gic.haoban.manage.api.dto.qdto.chat.OwnerSearchQDTO;
import com.gic.haoban.manage.api.enums.SecretTypeEnum;
import com.gic.haoban.manage.service.config.Config;
import com.gic.haoban.manage.service.dao.mapper.StaffMapper;
import com.gic.haoban.manage.service.dao.mapper.TabHaobanExternalClerkRelatedMapper;
import com.gic.haoban.manage.service.dao.mapper.chat.ChatNoticeMapper;
import com.gic.haoban.manage.service.dao.mapper.chat.GroupChatMapper;
import com.gic.haoban.manage.service.dao.mapper.chat.GroupChatOwnerHistoryMapper;
import com.gic.haoban.manage.service.dao.mapper.chat.GroupChatOwnerMapper;
import com.gic.haoban.manage.service.dao.mapper.chat.GroupChatUserMapper;
import com.gic.haoban.manage.service.entity.TabHaobanExternalClerkRelated;
import com.gic.haoban.manage.service.entity.TabHaobanStaff;
import com.gic.haoban.manage.service.entity.TabHaobanWxEnterprise;
import com.gic.haoban.manage.service.entity.chat.TabChatNotice;
import com.gic.haoban.manage.service.entity.chat.TabGroupChat;
import com.gic.haoban.manage.service.entity.chat.TabGroupChatOwner;
import com.gic.haoban.manage.service.entity.chat.TabGroupChatOwnerHistory;
import com.gic.haoban.manage.service.entity.chat.TabGroupChatUser;
import com.gic.haoban.manage.service.pojo.bo.chat.GroupChatBO;
import com.gic.haoban.manage.service.service.SecretSettingService;
import com.gic.haoban.manage.service.service.StaffClerkRelationService;
import com.gic.haoban.manage.service.service.WxEnterpriseService;
import com.gic.haoban.manage.service.service.chat.GroupChatService;
import com.gic.member.api.dto.MemberDTO;
import com.gic.member.api.service.MemberService;
import com.gic.wechat.api.dto.qywx.chat.GroupChatDetailDTO;
import com.gic.wechat.api.dto.qywx.chat.GroupChatDetailMemberDTO;
import com.gic.wechat.api.dto.qywx.chat.GroupChatListDTO;
import com.gic.wechat.api.dto.qywx.chat.GroupChatListItemDTO;
import com.gic.wechat.api.service.qywx.QywxChatApiService;
import com.github.pagehelper.PageHelper;

@Service("groupChatService")
public class GroupChatServiceImpl implements GroupChatService {

    private static final Logger logger = LoggerFactory.getLogger(GroupChatService.class);

    @Autowired
    private QywxChatApiService qywxChatApiService;
    @Autowired
    private GroupChatMapper groupChatMapper;
    @Autowired
    private GroupChatUserMapper groupChatUserMapper;
    @Autowired
    private WxEnterpriseService wxEnterpriseService;
    @Autowired
    private MemberService memberService;
    @Autowired
    private TabHaobanExternalClerkRelatedMapper externalClerkRelatedMapper;
    @Autowired
    private StaffMapper staffMapper;
    @Autowired
    private SecretSettingService secretSettingService;
    @Autowired
    private ChatNoticeMapper chatNoticeMapper;
    @Autowired
    private StaffClerkRelationService staffClerkRelationService;
    @Autowired
    private Config config;
    @Autowired
    private GroupChatOwnerMapper groupChatOwnerMapper;
    @Autowired
    private GroupChatOwnerHistoryMapper groupChatOwnerHistoryMapper;

    // 每次拉取成员数
    private int pageSize = 10;
    // 每次拉取群数
    private int limit = 1;

    @Override
    public void createChat(String corpid, String wxChatId) {
        String wxEnterpriseId = this.getWxEnterpriesId(corpid);
        if (null == wxEnterpriseId) {
            return;
        }
        WxEnterpriseQwDTO qwDTO = this.wxEnterpriseService.getQwInfo(wxEnterpriseId);
        this.addGroupChat(qwDTO, wxChatId, 0, false);
    }

    @Override
    public void dismissChat(String corpid, String wxChatId) {
        String wxEnterpriseId = this.getWxEnterpriesId(corpid);
        if (null == wxEnterpriseId) {
            return;
        }
        TabGroupChat groupChat = this.groupChatMapper.selectByWxChatId(wxEnterpriseId, wxChatId);
        if (null == groupChat) {
            logger.info("群查不到,wxChatid={}", wxChatId);
            return;
        }
        this.groupChatMapper.updateChatStatus(groupChat.getGroupChatId(), 4);
    }

    @Override
    public void updateChat(String corpid, String wxChatId, String updateDetail, String msg) {
        String wxEnterpriseId = this.getWxEnterpriesId(corpid);
        if (null == wxEnterpriseId) {
            return;
        }
        TabGroupChat groupChat = this.groupChatMapper.selectByWxChatId(wxEnterpriseId, wxChatId);
        if (null == groupChat) {
            logger.info("群不存在");
            return;
        }
        TabChatNotice entity = new TabChatNotice();
        entity.setId(UniqueIdUtils.uniqueLong());
        entity.setChatId(groupChat.getGroupChatId());
        entity.setWxEnterpriseId(wxEnterpriseId);
        entity.setMsg(msg);
        // change_name=群名变更change_notice=群公告变更 1
        if ("change_name".equals(updateDetail) || "change_notice".equals(updateDetail)) {
            entity.setType(1);
        }
        // change_owner=群主变更 2
        if ("change_owner".equals(updateDetail)) {
            entity.setType(2);
        }
        // add_member=成员入群 del_member=成员退群 3
        if ("add_member".equals(updateDetail) || "del_member".equals(updateDetail)) {
            entity.setType(3);
        }
        this.chatNoticeMapper.insert(entity);
    }

    @Override
    public void refreshChatInfo(Long groupChatId) {
        TabGroupChat chat = this.groupChatMapper.selectById(groupChatId);
        if (null == chat) {
            logger.info("群主变更事件,群不存在");
            return;
        }
        String wxEnterpriseId = chat.getWxEnterpriseId();
        WxEnterpriseQwDTO qwDTO = this.wxEnterpriseService.getQwInfo(wxEnterpriseId);
        this.updateGroupChatDetail(qwDTO, chat, true);
    }

    @Override
    public GroupChatBO detail(String wxEnterpriseId, Long groupChatId) {
        TabGroupChat groupChat = this.groupChatMapper.selectById(groupChatId);
        return EntityUtil.changeEntity(GroupChatBO.class, groupChat);
    }

    @Override
    public GroupChatBO getByWxChatId(String wxEnterpriseId, String wxChatId) {
        TabGroupChat groupChat = this.groupChatMapper.selectByWxChatId(wxEnterpriseId, wxChatId);
        return EntityUtil.changeEntity(GroupChatBO.class, groupChat);
    }

    @Override
    public GroupChatBO getByWxChatId3th(String wxEnterpriseId, String wxChatId3th) {
        TabGroupChat groupChat = this.groupChatMapper.selectByWxChatId3th(wxEnterpriseId, wxChatId3th);
        return EntityUtil.changeEntity(GroupChatBO.class, groupChat);
    }

    @Override
    public Page<GroupChatBO> listPage(GroupChatSearchQDTO qdto, BasePageInfo basePageInfo) {
        PageHelper.startPage(basePageInfo);
        List<TabGroupChat> list = this.groupChatMapper.list(qdto);
        Page<GroupChatBO> retPage = PageHelperUtils.changePageHelperToCurrentPage(list, GroupChatBO.class);
        return retPage;
    }

    @Override
    public void updateChatEnterpriseId(String wxEnterpriseId, String enterpriseId, List<Long> groupChatIdList) {

        this.groupChatMapper.updateChatEnterpriseId(wxEnterpriseId, enterpriseId, groupChatIdList);

        // 更新后重新同步群数据

    }

    @Override
    public void handChatNotice(String params) {
        List<TabChatNotice> list = this.chatNoticeMapper.listAll();
        this.chatNoticeMapper.closeAll();
        for (TabChatNotice item : list) {
            int type = item.getType();
            String wxEnterpriseId = item.getWxEnterpriseId();
            Long groupChatId = item.getChatId();
            WxEnterpriseQwDTO qwDTO = this.wxEnterpriseService.getQwInfo(wxEnterpriseId);
            if (type == 1) {
                this.updateGroupChatDetail(qwDTO, groupChatId, false);
            }
            if (type == 2) {
                this.updateChatStatus(qwDTO, groupChatId);
            }
            if (type == 3) {
                this.updateGroupChatDetail(qwDTO, groupChatId, true);
            }
        }
    }

    @Override
    public void initGroupChatForTimer(String params) {
        List<TabGroupChat> initList = this.groupChatMapper.listAllNeedInit();
        if (CollectionUtils.isNotEmpty(initList)) {
            logger.info("待初始化群数={}", initList.size());
            for (TabGroupChat item : initList) {
                String wxEnterpriseId = item.getWxEnterpriseId();
                Long groupChatId = item.getGroupChatId();
                WxEnterpriseQwDTO qwDTO = this.wxEnterpriseService.getQwInfo(wxEnterpriseId);
                this.updateGroupChatDetail(qwDTO, groupChatId, true);
            }
        }

        List<TabGroupChat> list = this.groupChatMapper.listAllNeedInitWxChatId3th();
        for (TabGroupChat item : list) {
            String wxEnterpriseId = item.getWxEnterpriseId();
            String staffId = item.getStaffId();
            this.updateWxChatId3th(wxEnterpriseId, staffId);
        }
    }

    // 更新群状态
    public void updateChatStatus(WxEnterpriseQwDTO qwDTO, Long groupChatId) {
        TabGroupChat chat = this.groupChatMapper.selectById(groupChatId);
        if (null == chat) {
            logger.info("群主变更事件,群不存在");
            return;
        }
        String staffId = chat.getStaffId();
        if (StringUtils.isBlank(staffId)) {
            logger.info("群主staffid不存在,groupChatId={}", groupChatId);
            return;
        }
        TabHaobanStaff staff = this.staffMapper.selectByPrimaryKey(staffId);
        if (null == staff) {
            logger.info("群主已删除,staffid={}", staffId);
            return;
        }
        String userId = staff.getWxUserId();
        if (qwDTO.needOpenUserIdDk()) {
            userId = staff.getWxOpenUseId();
        }
        this.syncGroupChatList(qwDTO, Arrays.asList(userId), null, false);
    }

    /**
     * 初始化整个群数据,拉取所有成员的所有群,并立即获取群信息、群成员、同时也会匹配客户标识
     */
    @Override
    public void initEnterpriseGroupChat(String wxEnterpriseId) {
        WxEnterpriseQwDTO qwDTO = this.wxEnterpriseService.getQwInfo(wxEnterpriseId);
        if (null == qwDTO) {
            logger.info("wxEnterpriseId不存在,id={}", wxEnterpriseId);
            return;
        }
        int pageNum = 0;
        while (true) {
            List<TabHaobanStaff> staffList = this.staffMapper.listAllByPage(wxEnterpriseId, pageNum * pageSize,
                    pageSize);
            if (CollectionUtils.isEmpty(staffList)) {
                break;
            }
            logger.info("查询staff表第{}页,共{}条", pageNum + 1, staffList.size());
            List<String> userIdList = null;
            if (qwDTO.needOpenUserIdDk()) {
                userIdList = staffList.stream().filter(dto -> StringUtils.isNotBlank(dto.getWxOpenUseId()))
                        .map(dto -> dto.getWxOpenUseId()).collect(Collectors.toList());
            } else {
                userIdList = staffList.stream().filter(dto -> StringUtils.isNotBlank(dto.getWxUserId()))
                        .map(dto -> dto.getWxUserId()).collect(Collectors.toList());
            }
            if (CollectionUtils.isNotEmpty(userIdList)) {
                this.syncGroupChatList(qwDTO, userIdList, null, true);
            }
            pageNum++;
        }
    }

    // 同步群列表
    private void syncGroupChatList(WxEnterpriseQwDTO qwDTO, List<String> userIdList, String cursor,
            boolean syncDetailFlag) {
        logger.info("qwDTO={},idlist={},cursor={}", JSON.toJSONString(qwDTO), JSON.toJSONString(userIdList), cursor);
        SecretSettingDTO secretSetting = secretSettingService.getSecretSetting(qwDTO.getWxEnterpriseId(),
                SecretTypeEnum.CUSTOMIZED_APP.getVal());
        if (null == secretSetting) {
            logger.info("未配置代开应用");
        }
        String secret = secretSetting.getSecretVal();
        ServiceResponse<GroupChatListDTO> resp = this.qywxChatApiService.groupchatList(qwDTO.getDkCorpid(), secret,
                userIdList, limit, cursor);
        if (!resp.isSuccess()) {
            logger.info("拉取群列表异常={}", JSON.toJSON(resp));
            return;
        }
        GroupChatListDTO dto = resp.getResult();
        cursor = dto.getNext_cursor();
        List<GroupChatListItemDTO> list = dto.getGroup_chat_list();
        for (GroupChatListItemDTO item : list) {
            String wxChatId = item.getChat_id();
            int chatStatus = item.getStatus();
            this.addGroupChat(qwDTO, wxChatId, chatStatus, syncDetailFlag);
        }
        // 从企微拉取下一批数据
        if (StringUtils.isNotBlank(cursor)) {
            this.syncGroupChatList(qwDTO, userIdList, cursor, syncDetailFlag);
        }
    }

    /**
     * 新增群记录, 如果是更新,只更新群状态,新增只记录群id和群状态，其他数据都在详情修改维护
     */
    private void addGroupChat(WxEnterpriseQwDTO qwDTO, String wxChatId, int chatStatus, boolean syncDetailFlag) {
        String wxEnterpriseId = qwDTO.getWxEnterpriseId();
        TabGroupChat groupChat = this.groupChatMapper.selectByWxChatId(wxEnterpriseId, wxChatId);
        Long groupChatId = null;
        if (null != groupChat) {
            groupChatId = groupChat.getGroupChatId();
            logger.info("更新群状态,wxchatid={},groupChatId={},新老状态={}", wxChatId, groupChatId, chatStatus,
                    groupChat.getChatStatus());
            if (groupChat.getChatStatus() != chatStatus) {
                this.groupChatMapper.updateChatStatus(groupChatId, chatStatus);
            }
        } else {
            groupChatId = UniqueIdUtils.uniqueLong();
            TabGroupChat chat = new TabGroupChat();
            chat.setGroupChatId(groupChatId);
            chat.setWxEnterpriseId(wxEnterpriseId);
            chat.setWxChatId(wxChatId);
            chat.setChatStatus(chatStatus);
            chat.setCreateTime(new Date());
            chat.setUpdateTime(new Date());
            chat.setGicFlag(0);
            chat.setDeleteFlag(0);
            chat.setTotalCount(0);
            if (syncDetailFlag) {
                // 标识初始化中,会在查询群详情处理
                chat.setInitFlag(2);
            } else {
                // 标识未初始化,由于异步，需要异步获取群详情
                chat.setInitFlag(0);
            }
            chat.setTotalMemberCount(0);
            chat.setAddMemberCount(0);
            chat.setAddCount(0);
            chat.setQuitCount(0);
            chat.setQuitMemberCount(0);
            this.groupChatMapper.insert(chat);
        }
        if (syncDetailFlag) {
            this.updateGroupChatDetail(qwDTO, groupChatId, true);
        }
    }

    private void updateGroupChatDetail(WxEnterpriseQwDTO qwDTO, TabGroupChat chat, boolean syncMember) {
        // 控制同时刷新
        Long groupChatId = chat.getGroupChatId();
        String wxEnterpriseId = chat.getWxEnterpriseId();
        String enterpriseId = chat.getEnterpriseId();
        SecretSettingDTO secretSetting = secretSettingService.getSecretSetting(wxEnterpriseId,
                SecretTypeEnum.CUSTOMIZED_APP.getVal());
        if (null == secretSetting) {
            logger.info("没有配置代开应用");
        }
        String secret = secretSetting.getSecretVal();
        ServiceResponse<GroupChatDetailDTO> detailResp = this.qywxChatApiService.groupchatDetail(qwDTO.getDkCorpid(),
                secret, chat.getWxChatId(), 1);

        GroupChatDetailDTO detail = detailResp.getResult();
        chat.setGroupChatId(groupChatId);
        chat.setChatAddTime(new Date(detail.getCreate_time() * 1000));
        chat.setChatNotice(detail.getNotice());
        chat.setName(detail.getName());
        // 处理群主
        String owner = detail.getOwner();
        TabHaobanStaff staff = this.staffMapper.selectByUserIdAndEnterpriseId(owner, wxEnterpriseId);
        String newStaffId = null;
        String oldStaffId = chat.getStaffId();
        if (null != staff) {
            // 更新staff
            newStaffId = staff.getStaffId();
            chat.setStaffId(staff.getStaffId());
        } else {
            chat.setStaffId(null);
            logger.info("群主未关联好办,不同步群成员,userid={}", owner);
        }
        if (StringUtils.isBlank(enterpriseId) && null != staff) {
            logger.info("开始匹配群的gic商户id,staffid={}", staff.getStaffId());
            List<StaffClerkRelationDTO> clerkList = this.staffClerkRelationService.listByStaffId(wxEnterpriseId,
                    staff.getStaffId());
            if (CollectionUtils.isNotEmpty(clerkList)) {
                Set<String> enterpriseIdSet = clerkList.stream().map(dto -> dto.getEnterpriseId())
                        .collect(Collectors.toSet());
                logger.info("成员关联导购数={},商户={}", clerkList.size(), enterpriseIdSet);
                if (enterpriseIdSet.size() == 1) {
                    enterpriseId = clerkList.get(0).getEnterpriseId();
                    chat.setEnterpriseId(enterpriseId);
                }
            }
        }
        logger.info("更新群信息={}", JSON.toJSONString(chat));
        if (StringUtils.isNotEmpty(chat.getEnterpriseId())) {
            chat.setGicFlag(1);
        }
        // 判断是否更新群主
        this.saveOwnerHistory(wxEnterpriseId, groupChatId, newStaffId, oldStaffId);
        chat.setOriginalStaffId(oldStaffId);
        this.groupChatMapper.update(chat);
        if (null == staff) {
            logger.info("不同步群成员信息,群主未关联好办,owner={}", owner);
            return;
        }
        this.saveOwner(wxEnterpriseId, staff.getStaffId());
        if (!syncMember) {
            logger.info("不同步群成员信息");
            return;
        }
        List<GroupChatDetailMemberDTO> memberList = detail.getMember_list();
        List<TabGroupChatUser> userList = new ArrayList<>();
        List<TabGroupChatUser> allUserList = this.groupChatUserMapper.listAllChatUser(wxEnterpriseId, groupChatId);
        Map<String, TabGroupChatUser> userMap = allUserList.stream()
                .collect(Collectors.toMap(TabGroupChatUser::getUserId, o -> o, ((k1, k2) -> k1)));
        for (GroupChatDetailMemberDTO member : memberList) {
            String userId = member.getUserid();
            TabGroupChatUser user = userMap.get(userId);
            if (null != user) {
                logger.info("更新用户");
                if (StringUtils.isEmpty(user.getMemberId())) {
                    this.matchGicMember(wxEnterpriseId, enterpriseId, member.getUnionid(), userId, user);
                }
                this.groupChatUserMapper.update(user);
                continue;
            }
            user = new TabGroupChatUser();
            user.setChatUserId(UniqueIdUtils.uniqueLong());
            user.setWxEnterpriseId(wxEnterpriseId);
            user.setEnterpriseId(enterpriseId);
            user.setUserType(member.getType());
            user.setJoinTime(new Date(member.getJoin_time() * 1000));
            user.setUserName(member.getName());
            user.setNickName(member.getGroup_nickname());
            user.setGroupChatId(groupChatId);
            user.setUserId(userId);
            user.setJoinScene(member.getJoin_scene());
            user.setJoinState(null);
            user.setUnionid(member.getUnionid());
            user.setDeleteFlag(0);
            user.setStatusFlag(1);
            user.setCreateTime(new Date());
            user.setUpdateTime(new Date());
            if (null != member.getInvitor()) {
                user.setInvitorUserId(member.getInvitor().getUserid());
            }
            this.matchGicMember(wxEnterpriseId, enterpriseId, member.getUnionid(), userId, user);
            userList.add(user);
        }
        List<String> nowUserIdList = memberList.stream().map(dto -> dto.getUserid()).collect(Collectors.toList());
        for (TabGroupChatUser item : userList) {
            this.groupChatUserMapper.insert(item);
        }
        // 删除退群的人
        List<Long> deleteIdList = new ArrayList<>();
        for (TabGroupChatUser user : allUserList) {
            if (!nowUserIdList.contains(user.getUserId())) {
                deleteIdList.add(user.getChatUserId());
            }
        }
        if (CollectionUtils.isNotEmpty(deleteIdList)) {
            this.groupChatUserMapper.dismiss(deleteIdList);
        }
        int addNum = userList.size();
        int delNum = deleteIdList.size();
        // 群统计
        // this.groupChatMapper.updateData();
    }

    private void saveOwnerHistory(String wxEnterpriseId, Long groupChatId, String newStaffId, String oldStaffId) {
        if (null == newStaffId && null == oldStaffId) {
            return;
        }
        if (null != newStaffId && null != oldStaffId && newStaffId.equals(oldStaffId)) {
            return;
        }
        if (null != newStaffId) {
            TabGroupChatOwnerHistory entity = new TabGroupChatOwnerHistory();
            entity.setOwnerHistoryId(UniqueIdUtils.uniqueLong());
            entity.setWxEnterpriseId(wxEnterpriseId);
            entity.setGroupChatId(groupChatId);
            entity.setStaffId(newStaffId);
            entity.setCreateTime(new Date());
            this.groupChatOwnerHistoryMapper.insert(entity);
        }
        if (null != oldStaffId) {
            this.groupChatOwnerHistoryMapper.updateOwnerEnd(groupChatId, oldStaffId);
        }
    }

    // 初始化群信息-不包括群状态
    private void updateGroupChatDetail(WxEnterpriseQwDTO qwDTO, Long groupChatId, boolean syncMember) {
        TabGroupChat chat = this.groupChatMapper.selectById(groupChatId);
        this.updateGroupChatDetail(qwDTO, chat, syncMember);
    }

    // 匹配群成员-客户
    private void matchGicMember(String wxEnterpriseId, String enterpriseId, String unionid, String externalUserId,
            TabGroupChatUser user) {
        if (StringUtils.isBlank(enterpriseId)) {
            return;
        }
        if (StringUtils.isNotBlank(unionid)) {
            MemberDTO member = this.memberService.getMemberByUnionid(unionid, enterpriseId);
            if (null != member) {
                user.setMemberId(member.getMemberId());
                user.setUserType(3);
            }
        } else if (StringUtils.isNotBlank(externalUserId)) {
            TabHaobanExternalClerkRelated related = this.externalClerkRelatedMapper
                    .getMemberForExternalId(wxEnterpriseId, externalUserId);
            if (null != related) {
                user.setMemberId(related.getMemberId());
                user.setUserType(3);
            }
        }
    }

    private String getWxEnterpriesId(String corpid) {
        TabHaobanWxEnterprise wxEnterprise = this.wxEnterpriseService.getEnterpriseBycorpId(corpid);
        if (wxEnterprise == null) {
            logger.info("企业不存在,corpid={}", corpid);
            return null;
        }
        return wxEnterprise.getWxEnterpriseId();
    }

    private void updateWxChatId3th(String wxEnterpriseId, String staffId) {
        WxEnterpriseQwDTO qwDTO = this.wxEnterpriseService.getQwInfo(wxEnterpriseId);
        TabHaobanStaff stff = this.staffMapper.selectByPrimaryKey(staffId);
        String userid = stff.getWxUserId();
        if (qwDTO.needOpenUserId3th()) {
            userid = stff.getWxOpenUseId();
        }
        this.updateWxChatId3th(qwDTO, Arrays.asList(userid), null, staffId);
    }

    private void updateWxChatId3th(WxEnterpriseQwDTO qwDTO, List<String> userIdList, String cursor, String staffId) {
        logger.info("qwDTO={},idlist={},cursor={}", JSON.toJSONString(qwDTO), JSON.toJSONString(userIdList), cursor);
        ServiceResponse<GroupChatListDTO> resp = this.qywxChatApiService.groupchatList3th(qwDTO.getThirdCorpid(),
                config.getWxSuiteid(), userIdList, limit, cursor);
        if (!resp.isSuccess()) {
            logger.info("拉取群异常={}", JSON.toJSON(resp));
            return;
        }
        GroupChatListDTO dto = resp.getResult();
        cursor = dto.getNext_cursor();
        List<GroupChatListItemDTO> list = dto.getGroup_chat_list();
        for (GroupChatListItemDTO item : list) {
            String wxChatId3th = item.getChat_id();
            // 控制同时刷新
            ServiceResponse<GroupChatDetailDTO> detailResp = this.qywxChatApiService
                    .groupchatDetail3th(qwDTO.getThirdCorpid(), config.getWxSuiteid(), wxChatId3th, 1);
            GroupChatDetailDTO detail = detailResp.getResult();
            Long createTimeLong = detail.getCreate_time() * 1000;
            String name = detail.getName();
            this.groupChatMapper.updateWxChatId3th(staffId, name, new Date(createTimeLong), wxChatId3th);
        }
        // 从企微拉取下一批数据
        if (org.apache.commons.lang3.StringUtils.isNotBlank(cursor)) {
            this.updateWxChatId3th(qwDTO, userIdList, cursor, staffId);
        }
    }

    private void saveOwner(String wxEnterpriseId, String staffId) {
        TabGroupChatOwner owner = this.groupChatOwnerMapper.selectByStaffId(staffId);
        int chatCount = this.groupChatMapper.getCountByStaffId(staffId);
        if (null != owner) {
            this.groupChatOwnerMapper.updateChatCount(staffId, chatCount);
        } else {
            owner = new TabGroupChatOwner();
            owner.setWxEnterpriseId(wxEnterpriseId);
            owner.setStaffId(staffId);
            owner.setChatCount(chatCount);
            this.groupChatOwnerMapper.insert(owner);
        }
    }

    @Override
    public Page<GroupChatOwnerDTO> listOwnerPage(OwnerSearchQDTO qdto, BasePageInfo basePageInfo) {
        PageHelper.startPage(basePageInfo);
        List<GroupChatOwnerDTO> list = this.groupChatOwnerMapper.listOwner(qdto);
        Page<GroupChatOwnerDTO> retPage = PageHelperUtils.changePageHelperToCurrentPage(list, GroupChatOwnerDTO.class);
        return retPage;
    }

    @Override
    public List<GroupChatOwnerHistoryDTO> listOwnerHistory(Long groupChatId, String WxEnterpriseId) {
        TabGroupChat chat = this.groupChatMapper.selectById(groupChatId);
        if (null == chat || org.apache.commons.lang3.StringUtils.isEmpty(chat.getStaffId())) {
            return null;
        }
        List<GroupChatOwnerHistoryDTO> list = this.groupChatOwnerHistoryMapper.listAll(groupChatId, WxEnterpriseId);
        for (GroupChatOwnerHistoryDTO item : list) {
            if (chat.getStaffId().equals(item.getStaffId())) {
                item.setEndTime(null);
            }
        }
        return list;
    }
}